// ENTITY_3DS Class    (c) by Heinrich Tillack 2002


#include "entity3ds.h"
#include "utils/3ds.h"

#include <GL/gl.h>
extern "C" {
	#include "texture.h"
}


GLvoid ENTITY_3DS::SetTextureDir(char * dir){

};


ENTITY_3DS::ENTITY_3DS(){

    model=NULL;
}

ENTITY_3DS::~ENTITY_3DS()
{
    if(model!=NULL){
        for(int i = 0; i < model->numOfObjects; i++)
        {
            // Free the faces, normals, vertices, and texture coordinates.
            delete [] model->pObject[i].pFaces;
            delete [] model->pObject[i].pNormals;
            delete [] model->pObject[i].pVerts;
            delete [] model->pObject[i].pTexVerts;
        }
    }
}

GLvoid ENTITY_3DS::SetModel( struct t3DModel *tmodel){
    model=tmodel;
}


GLvoid ENTITY_3DS::Draw()
{
    assert(model!=NULL);

    #define g_3Dmodel model


    // Since we know how many objects our model has, go through each of them.
    for(int i = 0; i < g_3Dmodel->numOfObjects; i++)
    {
        // Make sure we have valid objects just in case. (size() is in the vector class)
        if(g_3Dmodel->pObject.size() <= 0) break;

        // Get the current object that we are displaying
        t3DObject *pObject = &g_3Dmodel->pObject[i];

        // Check to see if this object has a texture map, if so bind the texture to it.
        if(pObject->bHasTexture) {



            // 			// Reset the color to normal again
            glColor3f(1, 1, 1);

            // Bind the texture map to the object by it's materialID

            switch_tex(g_3Dmodel->g_Texture[pObject->materialID]);


        } else {

            // Turn off texture mapping and turn on color
            glDisable(GL_TEXTURE_2D);

            // Reset the color to normal again
            glColor3f(1, 1, 1);
        }

        // This determines if we are in wireframe or normal mode
        glBegin(GL_TRIANGLES);

        // Go through all of the faces (polygons) of the object and draw them
        for(int j = 0; j < pObject->numOfFaces; j++)
        {
            // Go through each corner of the triangle and draw it.
            for(int whichVertex = 0; whichVertex < 3; whichVertex++)
            {
                // Get the index for each point of the face
                int index = pObject->pFaces[j].vertIndex[whichVertex];

                // Give OpenGL the normal for this vertex.
                //FIXME
                //glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);

                // If the object has a texture associated with it, give it a texture coordinate.
                if(pObject->bHasTexture) {

                    // Make sure there was a UVW map applied to the object or else it won't have tex coords.
                    if(pObject->pTexVerts) {
                        glTexCoord2f(pObject->pTexVerts[ index ].x, pObject->pTexVerts[ index ].y);
                    }
                } else {

                    // Make sure there is a valid material/color assigned to this object.
                    // You should always at least assign a material color to an object,
                    // but just in case we want to check the size of the material list.
                    // if the size is at least one, and the material ID != -1,
                    // then we have a valid material.
                    if(g_3Dmodel->pMaterials.size() && pObject->materialID >= 0)
                    {
                        // Get and set the color that the object is, since it must not have a texture
                        char *pColor = g_3Dmodel->pMaterials[pObject->materialID].color;

                        // Assign the current color to this model
                        //FIXME
                        //glColor3ub(pColor[0], pColor[1], pColor[2]);
                    }
                }

                // Pass in the current vertex of the object (Corner of current face)
                glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
            }
        }

        glEnd();								// End the drawing
    }

}
